=== modified file 'drizzled/display.cc'
--- drizzled/display.cc	2010-12-27 19:18:58 +0000
+++ drizzled/display.cc	2010-12-30 04:59:36 +0000
@@ -89,6 +89,12 @@
 static const std::string ITEM_CAST_CHAR("ITEM_CAST_CHAR");
 static const std::string ITEM_CAST_DECIMAL("ITEM_CAST_DECIMAL");
 
+static const std::string STRING_RESULT_STRING("STRING");
+static const std::string REAL_RESULT_STRING("REAL");
+static const std::string INT_RESULT_STRING("INTEGER");
+static const std::string ROW_RESULT_STRING("ROW");
+static const std::string DECIMAL_RESULT_STRING("DECIMAL");
+
 static const std::string YES("YES");
 static const std::string NO("NO");
 
@@ -203,6 +209,25 @@
   return PROGRAM_ERROR;
 }
 
+const std::string &type(Item_result type)
+{
+  switch (type)
+  {
+  case STRING_RESULT:
+    return STRING_RESULT_STRING;
+  case REAL_RESULT:
+    return REAL_RESULT_STRING;
+  case INT_RESULT:
+    return INT_RESULT_STRING;
+  case ROW_RESULT:
+    return ROW_RESULT_STRING;
+  case DECIMAL_RESULT:
+    return DECIMAL_RESULT_STRING;
+  }
+
+  assert(0);
+  return PROGRAM_ERROR;
+}
 
 const std::string &type(drizzled::enum_field_types type)
 {

=== modified file 'drizzled/display.h'
--- drizzled/display.h	2010-12-20 16:56:48 +0000
+++ drizzled/display.h	2010-12-30 04:59:36 +0000
@@ -30,9 +30,11 @@
 namespace display {
 
 const std::string &type(drizzled::Cast_target type);
+const std::string &type(Item_result type);
+const std::string &type(drizzled::enum_field_types type);
 const std::string &type(drizzled::enum_server_command type);
-const std::string &type(drizzled::enum_field_types type);
 const std::string &type(bool type);
+
 std::string hexdump(const unsigned char *str, size_t length);
 
 } /* namespace display */

=== modified file 'drizzled/enum.h'
--- drizzled/enum.h	2010-12-27 19:18:58 +0000
+++ drizzled/enum.h	2010-12-30 00:43:20 +0000
@@ -1066,6 +1066,8 @@
   ER_INVALID_CAST_TO_SIGNED,
   ER_INVALID_CAST_TO_UNSIGNED,
   ER_INVALID_BOOLEAN_VALUE,
+  ER_ASSERT,
+  ER_ASSERT_NULL,
   ER_INVALID_CAST_TO_BOOLEAN,
   
   // Leave ER_INVALID_BOOLEAN_VALUE as LAST, and force people to use tags

=== modified file 'drizzled/error.cc'
--- drizzled/error.cc	2010-12-27 19:18:58 +0000
+++ drizzled/error.cc	2010-12-30 02:37:34 +0000
@@ -513,6 +513,10 @@
   ADD_ERROR_MESSAGE(ER_INVALID_BOOLEAN_VALUE, N_("Received an invalid BOOLEAN value '%s'."));
   ADD_ERROR_MESSAGE(ER_INVALID_CAST_TO_BOOLEAN, N_("Invalid cast to BOOLEAN: '%s'."));
 
+  // ASSERT Message
+  ADD_ERROR_MESSAGE(ER_ASSERT, N_("Assertion '%s' failed."));
+  ADD_ERROR_MESSAGE(ER_ASSERT_NULL, N_("Assertion '%s' failed, the result was NULL."));
+
   // Some old error values use the same strings as some new error values.
   ADD_ERROR_MESSAGE(EE_FILENOTFOUND, find(ER_FILE_NOT_FOUND));
   ADD_ERROR_MESSAGE(EE_CANTCREATEFILE, find(ER_CANT_CREATE_FILE));

=== modified file 'drizzled/field/boolean.cc'
--- drizzled/field/boolean.cc	2010-12-26 22:31:33 +0000
+++ drizzled/field/boolean.cc	2010-12-30 04:58:34 +0000
@@ -170,7 +170,7 @@
   uint32_t mlength= (5) * cs->mbmaxlen;
 
   val_buffer->alloc(mlength);
-  char *buffer=(char*) val_buffer->ptr();
+  char *buffer=(char*) val_buffer->c_ptr();
 
   if (isTrue())
   {

=== modified file 'drizzled/function/cast/boolean.cc'
--- drizzled/function/cast/boolean.cc	2010-12-24 21:27:54 +0000
+++ drizzled/function/cast/boolean.cc	2010-12-30 04:58:34 +0000
@@ -52,7 +52,7 @@
 
       if (res->length() == 1)
       {
-        switch (res->ptr()[0])
+        switch (res->c_ptr()[0])
         {
         case 'y': case 'Y':
         case 't': case 'T': // PG compatibility
@@ -66,28 +66,28 @@
           break;
         }
       }
-      else if ((res->length() == 5) and (strcasecmp(res->ptr(), "FALSE") == 0))
-      {
-        return evaluate(false, value);
-      }
-      if ((res->length() == 4) and (strcasecmp(res->ptr(), "TRUE") == 0))
-      {
-        return evaluate(true, value);
-      }
-      else if ((res->length() == 5) and (strcasecmp(res->ptr(), "FALSE") == 0))
-      {
-        return evaluate(false, value);
-      }
-      else if ((res->length() == 3) and (strcasecmp(res->ptr(), "YES") == 0))
-      {
-        return evaluate(true, value);
-      }
-      else if ((res->length() == 2) and (strcasecmp(res->ptr(), "NO") == 0))
+      else if ((res->length() == 5) and (strcasecmp(res->c_ptr(), "FALSE") == 0))
+      {
+        return evaluate(false, value);
+      }
+      if ((res->length() == 4) and (strcasecmp(res->c_ptr(), "TRUE") == 0))
+      {
+        return evaluate(true, value);
+      }
+      else if ((res->length() == 5) and (strcasecmp(res->c_ptr(), "FALSE") == 0))
+      {
+        return evaluate(false, value);
+      }
+      else if ((res->length() == 3) and (strcasecmp(res->c_ptr(), "YES") == 0))
+      {
+        return evaluate(true, value);
+      }
+      else if ((res->length() == 2) and (strcasecmp(res->c_ptr(), "NO") == 0))
       {
         return evaluate(false, value);
       }
 
-      my_error(ER_INVALID_CAST_TO_BOOLEAN, MYF(0), res->ptr());
+      my_error(ER_INVALID_CAST_TO_BOOLEAN, MYF(0), res->c_ptr());
       return evaluate(false, value);
     }
 
@@ -113,7 +113,7 @@
   uint32_t mlength= (5) * cs->mbmaxlen;
 
   val_buffer->alloc(mlength);
-  char *buffer=(char*) val_buffer->ptr();
+  char *buffer=(char*) val_buffer->c_ptr();
 
   if (result)
   {
@@ -129,75 +129,6 @@
   return val_buffer;
 }
 
-#if 0
-bool Boolean::val_bool()
-{
-  bool tmp= false;
-
-  switch (args[0]->result_type())
-  {
-  case STRING_RESULT:
-    {
-      drizzled::String _res, *res;
-
-      if (not (res= args[0]->val_str(&_res)))
-      { 
-        null_value= true; 
-
-        break;
-      }
-      null_value= false; 
-
-      if (res->length() == 1)
-      {
-        switch (res->ptr()[0])
-        {
-        case 'y': case 'Y':
-        case 't': case 'T': // PG compatibility
-          return true;
-
-        case 'n': case 'N':
-        case 'f': case 'F': // PG compatibility
-          return false;
-
-        default:
-          break;
-        }
-      }
-      else if ((res->length() == 5) and (strcasecmp(res->ptr(), "FALSE") == 0))
-      {
-        return false;
-      }
-      if ((res->length() == 4) and (strcasecmp(res->ptr(), "TRUE") == 0))
-      {
-        return true;
-      }
-      else if ((res->length() == 5) and (strcasecmp(res->ptr(), "FALSE") == 0))
-      {
-        return false;
-      }
-      else if ((res->length() == 3) and (strcasecmp(res->ptr(), "YES") == 0))
-      {
-        return true;
-      }
-      else if ((res->length() == 2) and (strcasecmp(res->ptr(), "NO") == 0))
-      {
-        return false;
-      }
-
-      break;
-    }
-
-  default:
-    tmp= args[0]->val_bool();
-    null_value=args[0]->null_value;
-  }
-
-  return tmp;
-}
-#endif
-
-
 } // namespace cast
 } // namespace function
 } // namespace drizzled

=== modified file 'drizzled/sql_yacc.yy'
--- drizzled/sql_yacc.yy	2010-12-27 20:18:43 +0000
+++ drizzled/sql_yacc.yy	2010-12-30 06:23:25 +0000
@@ -3662,6 +3662,8 @@
           { $$=ITEM_CAST_SIGNED; Lex->charset= NULL; Lex->dec=Lex->length= (char*)0; }
         | SIGNED_SYM INT_SYM
           { $$=ITEM_CAST_SIGNED; Lex->charset= NULL; Lex->dec=Lex->length= (char*)0; }
+        | INT_SYM
+          { $$=ITEM_CAST_SIGNED; Lex->charset= NULL; Lex->dec=Lex->length= (char*)0; }
         | UNSIGNED_SYM
           { $$=ITEM_CAST_UNSIGNED; Lex->charset= NULL; Lex->dec=Lex->length= (char*)0; }
         | UNSIGNED_SYM INT_SYM

=== modified file 'plugin/debug/module.cc'
--- plugin/debug/module.cc	2010-12-27 20:20:52 +0000
+++ plugin/debug/module.cc	2010-12-30 00:43:20 +0000
@@ -56,8 +56,8 @@
     unsigned_flag= true;
   }
 
-  const char *func_name() const { return "assert"; }
-  const char *fully_qualified_func_name() const { return "assert()"; }
+  const char *func_name() const { return "assert_and_crash"; }
+  const char *fully_qualified_func_name() const { return "assert_and_crash()"; }
 
   bool val_bool()
   {
@@ -137,7 +137,7 @@
 
 static int initialize(drizzled::module::Context &context)
 {
-  context.add(new drizzled::plugin::Create_function<debug::Assert>("assert"));
+  context.add(new drizzled::plugin::Create_function<debug::Assert>("assert_and_crash"));
   context.add(new drizzled::plugin::Create_function<debug::Backtrace>("backtrace"));
   context.add(new drizzled::plugin::Create_function<debug::Crash>("crash"));
 

=== added file 'plugin/utility_functions/assert.cc'
--- plugin/utility_functions/assert.cc	1970-01-01 00:00:00 +0000
+++ plugin/utility_functions/assert.cc	2010-12-30 02:37:34 +0000
@@ -0,0 +1,57 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2010 Brian Aker
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include "config.h"
+
+#include "plugin/utility_functions/functions.h"
+#include <drizzled/error.h>
+
+namespace drizzled
+{
+
+namespace utility_functions
+{
+
+bool Assert::val_bool()
+{
+  if (not args[0]->val_bool())
+  {
+    drizzled::String res;
+    args[0]->print(&res, QT_ORDINARY);
+
+    assert(res.c_ptr());
+
+    if (args[0]->is_null())
+    {
+      drizzled::my_error(ER_ASSERT_NULL, MYF(0), res.c_ptr());
+    }
+    else
+    {
+      drizzled::my_error(ER_ASSERT, MYF(0), res.c_ptr());
+    }
+
+    return false;
+  }
+
+  return true;
+}
+
+} /* namespace utility_functions */
+} /* namespace drizzled */

=== added file 'plugin/utility_functions/assert.h'
--- plugin/utility_functions/assert.h	1970-01-01 00:00:00 +0000
+++ plugin/utility_functions/assert.h	2010-12-30 00:43:20 +0000
@@ -0,0 +1,65 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2010 Brian Aker
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+
+#ifndef PLUGIN_UTILITY_FUNCTIONS_ASSERT_H
+#define PLUGIN_UTILITY_FUNCTIONS_ASSERT_H
+
+#include <drizzled/item/function/boolean.h>
+#include <iostream>
+
+namespace drizzled
+{
+
+namespace utility_functions
+{
+
+class Assert :public drizzled::item::function::Boolean
+{
+public:
+  Assert() :
+    drizzled::item::function::Boolean()
+  {
+  }
+
+  bool val_bool();
+
+  int64_t val_int()
+  {
+    return val_bool();
+  }
+  const char *func_name() const { return "assert"; }
+  const char *fully_qualified_func_name() const { return "assert()"; }
+
+  void fix_length_and_dec()
+  {
+    max_length= 1;
+  }
+
+  bool check_argument_count(int n)
+  {
+    return (n == 1);
+  }
+};
+
+} /* namespace utility_functions */
+} /* namespace drizzled */
+
+#endif /* PLUGIN_UTILITY_FUNCTIONS_ASSERT_H */

=== modified file 'plugin/utility_functions/functions.cc'
--- plugin/utility_functions/functions.cc	2010-12-08 02:27:22 +0000
+++ plugin/utility_functions/functions.cc	2010-12-30 06:23:25 +0000
@@ -26,11 +26,14 @@
 
 static int init(drizzled::module::Context &context)
 {
+  context.add(new plugin::Create_function<utility_functions::Assert>("assert"));
   context.add(new plugin::Create_function<utility_functions::Catalog>("catalog"));
   context.add(new plugin::Create_function<utility_functions::Execute>("execute"));
   context.add(new plugin::Create_function<utility_functions::GlobalReadLock>("global_read_lock"));
+  context.add(new plugin::Create_function<utility_functions::ResultType>("result_type"));
   context.add(new plugin::Create_function<utility_functions::Kill>("kill"));
   context.add(new plugin::Create_function<utility_functions::Schema>("database"));
+  context.add(new plugin::Create_function<utility_functions::Typeof>("typeof"));
   context.add(new plugin::Create_function<utility_functions::User>("user"));
 
   return 0;
@@ -40,7 +43,7 @@
 {
   DRIZZLE_VERSION_ID,
   "Utility Functions",
-  "1.3",
+  "1.4",
   "Brian Aker, Stewart Smith",
   "Utility Functions.",
   PLUGIN_LICENSE_GPL,

=== modified file 'plugin/utility_functions/functions.h'
--- plugin/utility_functions/functions.h	2010-12-08 02:27:22 +0000
+++ plugin/utility_functions/functions.h	2010-12-30 06:23:25 +0000
@@ -24,11 +24,14 @@
 #include <drizzled/function/func.h>
 #include <drizzled/plugin/function.h>
 
+#include "plugin/utility_functions/assert.h"
 #include "plugin/utility_functions/catalog.h"
 #include "plugin/utility_functions/execute.h"
 #include "plugin/utility_functions/global_read_lock.h"
 #include "plugin/utility_functions/kill.h"
+#include "plugin/utility_functions/result_type.h"
 #include "plugin/utility_functions/schema.h"
+#include "plugin/utility_functions/typeof.h"
 #include "plugin/utility_functions/user.h"
 
 #endif /* PLUGIN_UTILITY_FUNCTIONS_FUNCTIONS_H */

=== modified file 'plugin/utility_functions/plugin.ini'
--- plugin/utility_functions/plugin.ini	2010-12-08 02:27:22 +0000
+++ plugin/utility_functions/plugin.ini	2010-12-30 06:23:25 +0000
@@ -3,19 +3,25 @@
 description= Utility Functions
 load_by_default=yes
 sources= 
+  assert.cc
   catalog.cc
   execute.cc
   functions.cc
   global_read_lock.cc
   kill.cc
+  result_type.cc
   schema.cc
+  typeof.cc
   user.cc
 headers=
+  assert.h
   catalog.h
   execute.h
   functions.h
   global_read_lock.h
   kill.h
+  result_type.h
   schema.h
+  typeof.h
   user.h
 static=yes

=== added file 'plugin/utility_functions/result_type.cc'
--- plugin/utility_functions/result_type.cc	1970-01-01 00:00:00 +0000
+++ plugin/utility_functions/result_type.cc	2010-12-30 04:59:36 +0000
@@ -0,0 +1,49 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2010 Brian Aker
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include "config.h"
+
+#include "plugin/utility_functions/functions.h"
+#include <drizzled/display.h>
+
+namespace drizzled
+{
+
+namespace utility_functions
+{
+
+String *ResultType::val_str(String *str)
+{
+  assert(fixed == 1);
+  null_value= false;
+
+  const std::string &tmp= display::type(args[0]->result_type());
+  str->copy(tmp.c_str(), tmp.length(), system_charset_info);
+
+  return str;
+}
+
+void ResultType::fix_length_and_dec()
+{
+  max_length= display::type(args[0]->result_type()).size() * system_charset_info->mbmaxlen;
+}
+
+} /* namespace utility_functions */
+} /* namespace drizzled */

=== added file 'plugin/utility_functions/result_type.h'
--- plugin/utility_functions/result_type.h	1970-01-01 00:00:00 +0000
+++ plugin/utility_functions/result_type.h	2010-12-30 04:59:36 +0000
@@ -0,0 +1,52 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2010 Brian Aker
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+
+#ifndef PLUGIN_UTILITY_FUNCTIONS_RESULT_TYPE_H
+#define PLUGIN_UTILITY_FUNCTIONS_RESULT_TYPE_H
+
+#include <drizzled/function/str/strfunc.h>
+
+namespace drizzled
+{
+
+namespace utility_functions
+{
+
+class ResultType :public Item_str_func
+{
+public:
+  ResultType()
+  { }
+
+  String *val_str(String *);
+  void fix_length_and_dec();
+  const char *func_name() const { return "result_type"; }
+  const char *fully_qualified_func_name() const { return "result_type()"; }
+
+  bool check_argument_count(int n)
+  {
+    return (n == 1);
+  }
+};
+
+} /* namespace utility_functions */
+} /* namespace drizzled */
+
+#endif /* PLUGIN_UTILITY_FUNCTIONS_RESULT_TYPE_H */

=== added file 'plugin/utility_functions/tests/r/assert.result'
--- plugin/utility_functions/tests/r/assert.result	1970-01-01 00:00:00 +0000
+++ plugin/utility_functions/tests/r/assert.result	2010-12-30 02:37:34 +0000
@@ -0,0 +1,66 @@
+SELECT ASSERT(1);
+ASSERT(1)
+1
+SELECT ASSERT(1+1);
+ASSERT(1+1)
+1
+SELECT ASSERT(TRUE);
+ASSERT(TRUE)
+1
+SELECT ASSERT(ASSERT(TRUE));
+ASSERT(ASSERT(TRUE))
+1
+SELECT ASSERT(1 > 0);
+ASSERT(1 > 0)
+1
+SELECT (SELECT 1) UNION SELECT (SELECT ASSERT(TRUE));
+(SELECT 1)
+1
+SELECT (SELECT ASSERT(TRUE)) UNION SELECT (SELECT ASSERT(TRUE));
+(SELECT ASSERT(TRUE))
+1
+SELECT (SELECT ASSERT(TRUE)) UNION SELECT (SELECT 1);
+(SELECT ASSERT(TRUE))
+1
+SELECT ASSERT(FALSE);
+ERROR HY000: Assertion 'FALSE' failed.
+SELECT ASSERT(ASSERT(FALSE));
+ERROR HY000: Assertion 'FALSE' failed.
+SELECT ASSERT(NULL);
+ERROR HY000: Assertion 'NULL' failed, the result was NULL.
+SELECT ASSERT(1 -1);
+ERROR HY000: Assertion '(1 - 1)' failed.
+SELECT ASSERT(1 < 0);
+ERROR HY000: Assertion '(1 < 0)' failed.
+SELECT (SELECT (SELECT (ASSERT(1 -1))));
+ERROR HY000: Assertion '(1 - 1)' failed.
+SELECT 1 FROM (SELECT ASSERT(FALSE)) failure;
+ERROR HY000: Assertion 'FALSE' failed.
+SELECT (SELECT 1) UNION SELECT (SELECT ASSERT(FALSE));
+ERROR HY000: Assertion 'FALSE' failed.
+SELECT (SELECT ASSERT(FALSE)) UNION SELECT (SELECT ASSERT(FALSE));
+ERROR HY000: Assertion 'FALSE' failed.
+SELECT (SELECT ASSERT(FALSE)) UNION SELECT (SELECT 1);
+ERROR HY000: Assertion 'FALSE' failed.
+select ASSERT(IF((0 > 1),1, NULL));
+ERROR HY000: Assertion 'if((0 > 1),1,NULL)' failed, the result was NULL.
+CREATE TABLE t1 (a int);
+SELECT ASSERT(COUNT(a)) FROM t1;
+ERROR HY000: Assertion 'count(`test`.`t1`.`a`)' failed.
+INSERT INTO t1 VALUES ();
+SELECT ASSERT(COUNT(*)) FROM t1;
+ASSERT(COUNT(*))
+1
+SELECT ASSERT(COUNT(a)) FROM t1;
+ERROR HY000: Assertion 'count(`test`.`t1`.`a`)' failed.
+DROP table t1;
+CREATE TABLE t1 AS SELECT ASSERT(TRUE);
+SELECT * from t1;
+ASSERT(TRUE)
+1
+DROP TABLE t1;
+CREATE TABLE t1 AS SELECT ASSERT(FALSE);
+ERROR HY000: Assertion 'FALSE' failed.
+SELECT ASSERT(COUNT(*) = 0) FROM DATA_DICTIONARY.TABLES WHERE TABLE_NAME="t1" AND TABLE_SCHEMA=SCHEMA();
+ASSERT(COUNT(*) = 0)
+1

=== added file 'plugin/utility_functions/tests/r/result_type.result'
--- plugin/utility_functions/tests/r/result_type.result	1970-01-01 00:00:00 +0000
+++ plugin/utility_functions/tests/r/result_type.result	2010-12-30 04:59:36 +0000
@@ -0,0 +1,21 @@
+SELECT RESULT_TYPE(NULL);
+RESULT_TYPE(NULL)
+STRING
+SELECT RESULT_TYPE(FALSE);
+RESULT_TYPE(FALSE)
+REAL
+SELECT RESULT_TYPE(TRUE);
+RESULT_TYPE(TRUE)
+REAL
+SELECT RESULT_TYPE("");
+RESULT_TYPE("")
+STRING
+SELECT RESULT_TYPE("110");
+RESULT_TYPE("110")
+STRING
+SELECT RESULT_TYPE(110);
+RESULT_TYPE(110)
+INTEGER
+SELECT RESULT_TYPE(1.1);
+RESULT_TYPE(1.1)
+DECIMAL

=== added file 'plugin/utility_functions/tests/r/typeof.result'
--- plugin/utility_functions/tests/r/typeof.result	1970-01-01 00:00:00 +0000
+++ plugin/utility_functions/tests/r/typeof.result	2010-12-30 06:23:25 +0000
@@ -0,0 +1,33 @@
+SELECT TYPEOF(NULL);
+TYPEOF(NULL)
+DRIZZLE_TYPE_NULL
+SELECT TYPEOF(FALSE);
+TYPEOF(FALSE)
+DRIZZLE_TYPE_DOUBLE
+SELECT TYPEOF(TRUE);
+TYPEOF(TRUE)
+DRIZZLE_TYPE_DOUBLE
+SELECT TYPEOF("");
+TYPEOF("")
+DRIZZLE_TYPE_VARCHAR
+SELECT TYPEOF("110");
+TYPEOF("110")
+DRIZZLE_TYPE_VARCHAR
+SELECT TYPEOF(110);
+TYPEOF(110)
+DRIZZLE_TYPE_LONGLONG
+SELECT TYPEOF(1.1);
+TYPEOF(1.1)
+DRIZZLE_TYPE_DECIMAL
+CREATE TABLE t1 ( A INT, B VARCHAR(10), C DOUBLE);
+INSERT INTO t1 VALUES ();
+SELECT TYPEOF(A) FROM t1;
+TYPEOF(A)
+DRIZZLE_TYPE_LONG
+SELECT TYPEOF(B) FROM t1;
+TYPEOF(B)
+DRIZZLE_TYPE_VARCHAR
+SELECT TYPEOF(c) FROM t1;
+TYPEOF(c)
+DRIZZLE_TYPE_DOUBLE
+DROP TABLE t1;

=== added file 'plugin/utility_functions/tests/t/assert.test'
--- plugin/utility_functions/tests/t/assert.test	1970-01-01 00:00:00 +0000
+++ plugin/utility_functions/tests/t/assert.test	2010-12-30 02:37:34 +0000
@@ -0,0 +1,70 @@
+SELECT ASSERT(1);
+SELECT ASSERT(1+1);
+SELECT ASSERT(TRUE);
+
+SELECT ASSERT(ASSERT(TRUE));
+
+SELECT ASSERT(1 > 0);
+
+SELECT (SELECT 1) UNION SELECT (SELECT ASSERT(TRUE));
+SELECT (SELECT ASSERT(TRUE)) UNION SELECT (SELECT ASSERT(TRUE));
+SELECT (SELECT ASSERT(TRUE)) UNION SELECT (SELECT 1);
+
+--error ER_ASSERT
+SELECT ASSERT(FALSE);
+
+--error ER_ASSERT
+SELECT ASSERT(ASSERT(FALSE));
+
+--error ER_ASSERT_NULL
+SELECT ASSERT(NULL);
+
+--error ER_ASSERT
+SELECT ASSERT(1 -1);
+
+--error ER_ASSERT
+SELECT ASSERT(1 < 0);
+
+--error ER_ASSERT
+SELECT (SELECT (SELECT (ASSERT(1 -1))));
+
+--error ER_ASSERT
+SELECT 1 FROM (SELECT ASSERT(FALSE)) failure;
+
+--error ER_ASSERT
+SELECT (SELECT 1) UNION SELECT (SELECT ASSERT(FALSE));
+
+--error ER_ASSERT
+SELECT (SELECT ASSERT(FALSE)) UNION SELECT (SELECT ASSERT(FALSE));
+
+--error ER_ASSERT
+SELECT (SELECT ASSERT(FALSE)) UNION SELECT (SELECT 1);
+
+--error ER_ASSERT_NULL
+select ASSERT(IF((0 > 1),1, NULL));
+
+
+# Look at assertions based on table result return
+
+CREATE TABLE t1 (a int);
+--error ER_ASSERT
+SELECT ASSERT(COUNT(a)) FROM t1;
+
+INSERT INTO t1 VALUES ();
+SELECT ASSERT(COUNT(*)) FROM t1;
+
+--error ER_ASSERT
+SELECT ASSERT(COUNT(a)) FROM t1;
+
+DROP table t1;
+
+# Look at assertions based on assert during create table
+
+CREATE TABLE t1 AS SELECT ASSERT(TRUE);
+SELECT * from t1;
+DROP TABLE t1;
+
+--error ER_ASSERT
+CREATE TABLE t1 AS SELECT ASSERT(FALSE);
+
+SELECT ASSERT(COUNT(*) = 0) FROM DATA_DICTIONARY.TABLES WHERE TABLE_NAME="t1" AND TABLE_SCHEMA=SCHEMA();

=== added file 'plugin/utility_functions/tests/t/result_type.test'
--- plugin/utility_functions/tests/t/result_type.test	1970-01-01 00:00:00 +0000
+++ plugin/utility_functions/tests/t/result_type.test	2010-12-30 04:59:36 +0000
@@ -0,0 +1,7 @@
+SELECT RESULT_TYPE(NULL);
+SELECT RESULT_TYPE(FALSE);
+SELECT RESULT_TYPE(TRUE);
+SELECT RESULT_TYPE("");
+SELECT RESULT_TYPE("110");
+SELECT RESULT_TYPE(110);
+SELECT RESULT_TYPE(1.1);

=== added file 'plugin/utility_functions/tests/t/typeof.test'
--- plugin/utility_functions/tests/t/typeof.test	1970-01-01 00:00:00 +0000
+++ plugin/utility_functions/tests/t/typeof.test	2010-12-30 06:23:25 +0000
@@ -0,0 +1,23 @@
+SELECT TYPEOF(NULL);
+SELECT TYPEOF(FALSE);
+SELECT TYPEOF(TRUE);
+SELECT TYPEOF("");
+SELECT TYPEOF("110");
+SELECT TYPEOF(110);
+SELECT TYPEOF(1.1);
+
+CREATE TABLE t1 ( A INT, B VARCHAR(10), C DOUBLE);
+INSERT INTO t1 VALUES ();
+SELECT TYPEOF(A) FROM t1;
+SELECT TYPEOF(B) FROM t1;
+SELECT TYPEOF(c) FROM t1;
+DROP TABLE t1;
+
+# todo -> make this work
+# SELECT CAST(TYPEOF(NULL) AS INTEGER);
+# SELECT CAST(TYPEOF(FALSE) AS INTEGER);
+# SELECT CAST(TYPEOF(TRUE) AS INTEGER);
+# SELECT CAST(TYPEOF("") AS INTEGER);
+# SELECT CAST(TYPEOF("110") AS INTEGER);
+# SELECT CAST(TYPEOF(110) AS INTEGER);
+# SELECT CAST(TYPEOF(1.1) AS INTEGER);

=== added file 'plugin/utility_functions/typeof.cc'
--- plugin/utility_functions/typeof.cc	1970-01-01 00:00:00 +0000
+++ plugin/utility_functions/typeof.cc	2010-12-30 06:23:25 +0000
@@ -0,0 +1,54 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2010 Brian Aker
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include "config.h"
+
+#include "plugin/utility_functions/functions.h"
+#include <drizzled/display.h>
+
+namespace drizzled
+{
+
+namespace utility_functions
+{
+
+int64_t Typeof::val_int()
+{
+  return static_cast<int64_t>(args[0]->field_type());
+}
+
+String *Typeof::val_str(String *str)
+{
+  assert(fixed == 1);
+  null_value= false;
+
+  const std::string &tmp= display::type(args[0]->field_type());
+  str->copy(tmp.c_str(), tmp.length(), system_charset_info);
+
+  return str;
+}
+
+void Typeof::fix_length_and_dec()
+{
+  max_length= display::type(args[0]->field_type()).size() * system_charset_info->mbmaxlen;
+}
+
+} /* namespace utility_functions */
+} /* namespace drizzled */

=== added file 'plugin/utility_functions/typeof.h'
--- plugin/utility_functions/typeof.h	1970-01-01 00:00:00 +0000
+++ plugin/utility_functions/typeof.h	2010-12-30 06:23:25 +0000
@@ -0,0 +1,54 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2010 Brian Aker
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+
+#ifndef PLUGIN_UTILITY_FUNCTIONS_TYPEOF_H
+#define PLUGIN_UTILITY_FUNCTIONS_TYPEOF_H
+
+#include <drizzled/function/str/strfunc.h>
+
+namespace drizzled
+{
+
+namespace utility_functions
+{
+
+class Typeof :public Item_str_func
+{
+public:
+  Typeof()
+  { }
+
+  int64_t val_int();
+
+  String *val_str(String *);
+  void fix_length_and_dec();
+  const char *func_name() const { return "typeof"; }
+  const char *fully_qualified_func_name() const { return "typeof()"; }
+
+  bool check_argument_count(int n)
+  {
+    return (n == 1);
+  }
+};
+
+} /* namespace utility_functions */
+} /* namespace drizzled */
+
+#endif /* PLUGIN_UTILITY_FUNCTIONS_TYPEOF_H */

